pub struct PreparedQuery<Q: Query> { /* private fields */ }Expand description
A prepared query can be stored independently of the World to amortize query set-up costs.
Implementations
Sourceimpl<Q: Query> PreparedQuery<Q>
impl<Q: Query> PreparedQuery<Q>
Sourcepub fn query<'q>(&'q mut self, world: &'q World) -> PreparedQueryBorrow<'q, Q>
pub fn query<'q>(&'q mut self, world: &'q World) -> PreparedQueryBorrow<'q, Q>
Query world, using dynamic borrow checking
This will panic if it would violate an existing unique reference or construct an invalid unique reference.